@@ -82,10 +82,6 @@
 # include <gdk/gdkx.h>
 #endif
 
-#ifdef USE_HILDON_EXTRAS
-#	include <hildon-extras/he-menu-view.h>
-#endif /* USE_HILDON_EXTRAS */
-
 #ifdef HAVE_STARTUP_NOTIFICATION
 static SnLauncheeContext *sn_context = NULL;
 static SnDisplay *sn_display = NULL;
@@ -560,7 +556,7 @@ int main(int argc, char *argv[])
 	struct option long_options[] = {
 		{"config",       required_argument, NULL, 'c'},
 		{"debug",        no_argument,       NULL, 'd'},
-		{"force-online", no_argument,       NULL, 'd'},
+		{"force-online", no_argument,       NULL, 'f'},
 		{"help",         no_argument,       NULL, 'h'},
 		{"login",        optional_argument, NULL, 'l'},
 		{"multiple",     no_argument,       NULL, 'm'},
